unit xAbout;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Buttons, XrButton, ExtCtrls, Animate, StdCtrls, LblEffct;

type
  TAboutRec = record
    Version: String;
    Author: String;
  end;
  TAboutDlg = class(TForm)
    Box: TPaintBox;
    Timer1: TTimer;
    Image1: TImage;
    Image: TAnimatedImage;
    Btn: TXRSpeedButton;
    Image2: TImage;
    lbAuthor: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    lbVersion: TLabelEffect;
    procedure FormCreate(Sender: TObject);
    procedure BtnClick(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure BoxPaint(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
  private
    Colors: Array[1..255] of TColor;
    Bmp,Pic: TBitmap;
    D: Integer;
    W: Integer;
    IsCreate: Boolean;
    procedure Draw;
    procedure NewColor(n: Integer);
    procedure CreateBox;

    procedure Frame(ARect: TRect;C1,C2: TColor);

    procedure WMNCHitTest(var M: TWMNCHitTest); message wm_NCHitTest;
  public
  end;

procedure ExecAboutDlg(AboutRec: TAboutRec);

var
  AboutDlg: TAboutDlg;
  
implementation

{$R *.DFM}
const
  R = 40;
  Num = 7;

procedure ExecAboutDlg(AboutRec: TAboutRec);
var
  AboutDlg: TAboutDlg;
Begin
  AboutDlg:=TAboutDlg.Create(Application);
  With AboutDlg Do
  try
    lbVersion.Caption:=AboutRec.Version;
    lbAuthor.Caption:=AboutRec.Author;
    ShowModal;
  finally
    Free;
  end;
End;

procedure TAboutDlg.Frame(ARect: TRect;C1,C2: TColor);
Begin
  With Image1.Canvas, ARect do
  begin
    Pen.Color := C1;
    Arc(Left,Top,Left+R,Top+R,Left+R div 2,Top,Left,Top+R div 2);
    MoveTo(Left,R div 2);
    LineTo((Right+Left) div 2,Bottom);
    MoveTo(R div 2,Top);
    LineTo(Right-R div 2,Top);
    Pen.Color := C2;
    Arc(Right-R,Top,Right,Top+R,Right,Top+R div 2,Right-R div 2,Top);
    MoveTo(Right,Top+R div 2);
    LineTo((Right+Left) div 2,Bottom);

    InflateRect(ARect,-1,-1);
    Pen.Color := C2;
    Arc(Left,Top,Left+R,Top+R,Left+R div 2,Top,Left,Top+R div 2);
    MoveTo(Left,R div 2);
    LineTo((Right+Left) div 2,Bottom-2);
    MoveTo(R div 2,Top);
    LineTo(Right-R div 2,Top);
    Pen.Color := C1;
    Arc(Right-R,Top,Right,Top+R,Right,Top+R div 2,Right-R div 2,Top);
    MoveTo(Right-1,Top+R div 2);
    LineTo((Right+Left) div 2,Bottom-2);

  end;
End;

procedure TAboutDlg.FormCreate(Sender: TObject);
var
  rgTop,rgMiddle,rgMinus: HRGN;
  A: Array[0..2] of TPoint;
begin
  rgTop := CreateRoundRectRgn(0,0, Width, R, R, R);
  rgMinus := CreateRectRgn(0,0,Width, R div 2);

  A[0].X := 0;          A[0].Y := R div 2;
  A[1].X := Width div 2;A[1].Y := Height;
  A[2].X := Width;      A[2].Y := R div 2;
  rgMiddle := CreatePolygonRgn(A,3, WINDING);

  CombineRgn(rgTop,rgTop,rgMinus,RGN_AND);
  CombineRgn(rgTop,rgTop,rgMiddle,RGN_OR);
  SetWindowRgn(Handle,rgTop,True);
  DeleteObject(rgMiddle);
  DeleteObject(rgMinus);

  CreateBox;
  Image1.Visible := True;
  FormPaint(Nil);
end;

procedure TAboutDlg.BtnClick(Sender: TObject);
begin
  Close;
end;

procedure TAboutDlg.FormPaint(Sender: TObject);
var
  ARect: TRect;
Begin
  ARect := ClientRect;
  InflateRect(ARect,-1,-1);
  Frame(ARect,clWhite,clGray);
End;

procedure TAboutDlg.Draw;
var
  n,X: Integer;
  procedure DrawLine(X,W,n: Integer);
  var
    i,Y: Integer;
  begin
    IF n>255 Then Exit;
    Pic.Canvas.Pen.Color := Colors[n];
    For i := 0 to W-1 do
    with Pic.Canvas do
    begin
      IF X+i>Pic.Width-1 Then Exit;
      MoveTo(X+i,0);
      LineTo(X+i+W,Pic.Height);
    end;
  end; // DrawLine
begin // Draw
  With Pic.Canvas,Pic do
  begin
    Draw(0,0,Bmp);
    Pen.Mode := pmMask;
    DrawLine(-Bmp.Height-W,D,1);
    n := 1;
    X := -Bmp.Height-W+D;
    while X<Width do
    begin
      Inc(n);
      DrawLine(X,W,n);
      Inc(X,W);
    end;
  end; // With
end;

procedure TAboutDlg.Timer1Timer(Sender: TObject);
var
  i,j: Integer;
  Tmp: TBitmap;
Begin
  Draw;
  Inc(D);
  IF D>=W Then
  begin
    For i := 255 downto 2 do
      Colors[i] := Colors[i-1];
    NewColor(1);
    D := 0;
  end;
  Tmp := TBitmap.Create;
  Tmp.Width := Box.Width;
  Tmp.Height := Box.Height;
  Tmp.Canvas.CopyRect(Rect(0,0,Pic.Width,Pic.Height),Image1.Canvas,Box.BoundsRect);
  Pic.Transparent := True;
  Pic.TransparentColor := clBlack;
  Tmp.Canvas.Draw(0,0,Pic);
  Canvas.Draw(Box.Left,Box.Top,Tmp);
  Tmp.Free;
End;

procedure TAboutDlg.NewColor(n: Integer);
const
  Dc=64;
  Rc=128+64;
//  C: Array[0..3] of TColor=(clRed,clLime,clBlue,clYellow);
Begin
  Colors[n] := RGB(Random(Rc)+Dc,Random(Rc)+Dc,Random(Rc)+Dc);
//  Colors[n] := C[Random(4)];
End;

procedure TAboutDlg.BoxPaint(Sender: TObject);
begin
  IF IsCreate Then Exit;
  Box.Canvas.Draw(0,0,Pic);
end;

procedure TAboutDlg.CreateBox;
var
  i: Integer;
begin
  IsCreate := True;
  Bmp := TBitmap.Create;
  Pic := TBitmap.Create;
  Bmp.Width := Box.Width;
  Bmp.Height := Box.Height;
  Pic.Width := Box.Width;
  Pic.Height := Box.Height;
  Color := clBlack;
  Bmp.Canvas.CopyRect(
    Rect(0,0,Bmp.Width,Bmp.Height),
    GetFormImage.Canvas,
    Box.BoundsRect);
  Color := clBtnFace;
  D := 0;
  W := Box.Width div Num;
  For i := 1 to 255 do
    NewColor(i);
  Draw;
  IsCreate := False;
end;

procedure TAboutDlg.FormDestroy(Sender: TObject);
begin
  Bmp.Free;
  Pic.Free;
end;

procedure TAboutDlg.WMNCHitTest(var M: TWMNCHitTest);
begin
  inherited;
  if  (M.Result = htClient) and (not MouseInControl(Btn)) then
    M.Result := htCaption;

end;

procedure TAboutDlg.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  IF (Key=VK_RETURN) or (Key=VK_ESCAPE) Then
    Close;
end;




end.
