{*******************************************************}
{                                                       }
{         Delphi VCL eXtensions   (X)                   }
{                                                       }
{         Copyright (c) 1997 Alexander Buzaev           }
{                                                       }
{*******************************************************}

unit xDBTree;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  CommCtrl, ComCtrls, DB, DBTables, TypInfo, DsgnIntf;

type
  TNodeEvent = procedure (Sender: TObject; Node: TTreeNode) of object;
  TXTreeNode = Class;
  TXDBTreeDataLink = class;
  TXCustomDBTreeView = class(TTreeView)
  private
    FUpdateLock: Boolean;
    FDataFields: array[0..3] of String;
    FOnCreateNode: TNodeEvent;
    FOnFillNode: TNodeEvent;
    FOnDestroyNode: TNodeEvent;
    FOnBuildTree: TNotifyEvent;
    procedure SetNode(Index: integer; ANode: TXTreeNode);
    function GetNode(Index: integer): TXTreeNode;
    function GetDataSource: TDataSource;
    procedure SetDataSource(Value: TDataSource);
    procedure SetDataField(Index: Integer; Value: String);
    function GetField(Index: Integer): TField;
  protected
    FDataLink: TXDBTreeDataLink;
    FNodes: TList;
    iSaveAbsoluteIndex: Integer;
    procedure DoCreateNode(Node: TTreeNode); dynamic;
    procedure DoDestroyNode(Node: TTreeNode); dynamic;
    procedure DoFillNode(Node: TTreeNode); dynamic;
    procedure DoBuildTree; dynamic;
    procedure BuildNode(Node: TXTreeNode); virtual;
    function  CreateNode: TTreeNode; override;
    procedure CreateWnd; override;
    procedure DestroyWnd; override;
    procedure SaveState; virtual;
    procedure LoadState; virtual;
    procedure SaveTree; virtual;
    procedure ClearNodes;

    property OnCreateNode: TNodeEvent read FOnCreateNode write FOnCreateNode;
    property OnFillNode: TNodeEvent read FOnFillNode write FOnFillNode;
    property OnBuildTree: TNotifyEvent read FOnBuildTree write FOnBuildTree;
    property OnDestroyNode: TNodeEvent read FOnDestroyNode write FOnDestroyNode;

    property Nodes[Index : Integer]: TXTreeNode read GetNode write SetNode; default;
    property DataSource: TDataSource read GetDataSource write SetDataSource;
    property DataFieldID: String index 0 read FDataFields[0] write SetDataField;
    property DataFieldOwnerID: String index 1 read FDataFields[1] write SetDataField;
    property DataFieldText: String index 2 read FDataFields[2] write SetDataField;
    property DataFieldImageIndex: String index 3 read FDataFields[3] write SetDataField;
    property FieldID: TField index 0 read GetField;
    property FieldOwnerID: TField index 1 read GetField;
    property FieldText: TField index 2 read GetField;
    property FieldImageIndex: TField index 3 read GetField;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure RefreshTree; virtual;
    procedure Loaded; override;
    function GetLastNodeAtLevel(Node: TTreeNode; Level: Integer): TTreeNode; virtual;
    function GetNodeAtID(ID: Extended): TTreeNode; virtual;
  published
  end;

  //      TTreeNode.Destoy  (  owner    Nil)
  TxNodeInfo=record
    ID, OwnerID: Integer;
    ImageIndex: Integer;
    Data: Pointer;
    Text: String;
  end;

  TxTreeNode = class (TTreeNode)
  private
  public
    ID, OwnerID: Integer;
    procedure GetNodeInfo(var InfoRec: TxNodeInfo);
    constructor Create(AOwner: TTreeNodes);
  end;

  TXDBTreeView = class(TXCustomDBTreeView)
  public
    // new properties
    property FieldID;
    property FieldOwnerID;
    property FieldText;
    //inherited propertied
    property Items;
  published
    //new properties
    property DataSource;
    property DataFieldID;
    property DataFieldOwnerID;
    property DataFieldImageIndex;
    property DataFieldText;

    property OnBuildTree;
    property OnFillNode;
    //inherited propertied
    property ShowButtons;
    property BorderStyle;
    property DragCursor;
    property ShowLines;
    property ShowRoot;
    property ReadOnly;
    property DragMode;
    property HideSelection;
    property Indent;
    property OnEditing;
    property OnEdited;
    property OnExpanding;
    property OnExpanded;
    property OnCollapsing;
    property OnCompare;
    property OnCollapsed;
    property OnChanging;
    property OnChange;
    property OnDeletion;
    property OnGetImageIndex;
    property OnGetSelectedIndex;
    property Align;
    property Enabled;
    property Font;
    property Color;
    property ParentColor;
    property ParentCtl3D;
    property Ctl3D;
    property SortType;
    property TabOrder;
    property TabStop default True;
    property Visible;
    property OnClick;
    property OnEnter;
    property OnExit;
    property OnDragDrop;
    property OnDragOver;
    property OnStartDrag;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnDblClick;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property PopupMenu;
    property ParentFont;
    property ParentShowHint;
    property ShowHint;
    property Images;
    property StateImages;
  end;

  TXDBTreeDataLink = class (TDataLink)
  private
    FTreeView: TXCustomDBTreeView;
  protected
    procedure ActiveChanged; override;
  end;

procedure Register;

implementation
{$R *.DCR}

type
  TDBStringProperty = class(TStringProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    procedure GetValueList(List: TStrings); virtual; abstract;
    procedure GetValues(Proc: TGetStrProc); override;
  end;
  TDataFieldProperty = class(TDBStringProperty)
  public
    procedure GetValueList(List: TStrings); override;
    function GetDataSourcePropName: string; virtual;
  end;

//------------------- TDBStringProperty ---------------------------------------------
function TDBStringProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paValueList, paSortList, paMultiSelect];
end;

procedure TDBStringProperty.GetValues(Proc: TGetStrProc);
var
  I: Integer;
  Values: TStringList;
begin
  Values := TStringList.Create;
  try
    GetValueList(Values);
    for I := 0 to Values.Count - 1 do Proc(Values[I]);
  finally
    Values.Free;
  end;
end;

//------------------- TDataFieldProperty ---------------------------------------------
function TDataFieldProperty.GetDataSourcePropName: string;
begin
  Result := 'DataSource';
end;

procedure TDataFieldProperty.GetValueList(List: TStrings);
var
  Instance: TComponent;
  PropInfo: PPropInfo;
  DataSource: TDataSource;
begin
  Instance := TComponent(GetComponent(0));
  PropInfo := TypInfo.GetPropInfo(Instance.ClassInfo, GetDataSourcePropName);
  if (PropInfo <> nil) and (PropInfo^.PropType^.Kind = tkClass) then
  begin
    DataSource := TObject(GetOrdProp(Instance, PropInfo)) as TDataSource;
    if (DataSource <> nil) and (DataSource.DataSet <> nil) then
      DataSource.DataSet.GetFieldNames(List);
  end;
end;

procedure Register;
begin
  RegisterComponents('X', [TXDBTreeView]);
  RegisterPropertyEditor(TypeInfo(String), TXDBTreeView,
    'DataFieldID', TDataFieldProperty);
  RegisterPropertyEditor(TypeInfo(String), TXDBTreeView,
    'DataFieldOwnerID', TDataFieldProperty);
  RegisterPropertyEditor(TypeInfo(String), TXDBTreeView,
    'DataFieldText', TDataFieldProperty);
  RegisterPropertyEditor(TypeInfo(String), TXDBTreeView,
    'DataFieldImageIndex', TDataFieldProperty);
end;

// TxTreeNode
constructor TXTreeNode.Create(AOwner: TTreeNodes);
Begin
  inherited;
  ID:=-1; OwnerID:=-1;
End;

procedure TXTreeNode.GetNodeInfo(var InfoRec: TxNodeInfo);
Begin
  InfoRec.ID:=ID;
  InfoRec.OwnerID:=OwnerID;
  InfoRec.ImageIndex:=ImageIndex;
  InfoRec.Data:=Data;
  InfoRec.Text:=Text;
End;

// TXDBTreeDataLink
procedure TXDBTreeDataLink.ActiveChanged;
begin
  FTreeView.DoBuildTree;
end;

// TXCustomDBTreeView
constructor TXCustomDBTreeView.Create(AOwner: TComponent);
begin
  inherited;
  FDataLink := TXDBTreeDataLink.Create;
  FDataLink.FTreeView := Self;
  FNodes := TList.Create;
  FUpdateLock:=False;
  OnChange:=Nil;
end;

destructor TXCustomDBTreeView.Destroy;
begin
  SetDataSource(nil);
  ClearNodes;
  FDataLink.Free;
  FNodes.Free;
  inherited;
end;

procedure TXCustomDBTreeView.Loaded;
Begin
  inherited;
end;

procedure TXCustomDBTreeView.ClearNodes;
var
  i: Integer;
Begin
  For i:=0 To FNodes.Count-1 Do
    Nodes[i].Free;
  FNodes.Clear;
End;

function TXCustomDBTreeView.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

function TXCustomDBTreeView.GetField(Index: Integer): TField;
begin
  Result := nil;
  if FDataLink.Active then
    Result := DataSource.DataSet.FieldByName(FDataFields[Index]);
end;

procedure TXCustomDBTreeView.SetDataField(Index: Integer; Value: String);
begin
  if (FDataFields[Index] <> Value) then
  begin
    FDataFields[Index] := Value;
    DoBuildTree;
  end;
end;

procedure TXCustomDBTreeView.SetDataSource(Value: TDataSource);
begin
  FDataLink.DataSource := Value;
  if Assigned(Value) then FreeNotification(Value);
end;

procedure TXCustomDBTreeView.DoDestroyNode(Node: TTreeNode);
begin
  if Assigned(FOnDestroyNode) then FOnDestroyNode(Self, Node);
end;

procedure TXCustomDBTreeView.DoCreateNode(Node: TTreeNode);
begin
  if Assigned(FOnCreateNode) then FOnCreateNode(Self, Node);
end;

procedure TXCustomDBTreeView.DoFillNode(Node: TTreeNode);
begin
  if Assigned(FOnFillNode) then FOnFillNode(Self, Node);
end;

function TXCustomDBTreeView.CreateNode: TTreeNode;
begin
  Result := TXTreeNode.Create(Items);
end;

procedure TXCustomDBTreeView.SetNode(Index: Integer; ANode: TXTreeNode);
Begin
  FNodes[Index]:=ANode;
End;

function TXCustomDBTreeView.GetNode(Index: Integer): TXTreeNode;
Begin
  Result:=FNodes[Index];
End;

function TXCustomDBTreeView.GetNodeAtID(ID: Extended): TTreeNode;
var
  i: Integer;
Begin
  Result:=Nil;
  For i:=0 To Items.Count-1 Do
    IF (Items[i] as TXTreeNode).ID=ID Then
    begin
      Result:=Items[i];
      Break;
    end;
End;

function TXCustomDBTreeView.GetLastNodeAtLevel(Node: TTreeNode; Level: Integer): TTreeNode;
var
  i: Integer;
  t: TTreeNode;
Begin
  i := 0;
  t := Node;
  while (i < Level-1) and (t <> nil) and (t.GetLastChild <> nil) do
  begin
    inc(i);
    t := t.GetLastChild;
  end;
  result := t;
End;


procedure TXCustomDBTreeView.SaveState;
Begin
  iSaveAbsoluteIndex:=0;
  IF Selected<>Nil Then
    iSaveAbsoluteIndex := Selected.AbsoluteIndex;
End;

procedure TXCustomDBTreeView.LoadState;
Begin
  IF (iSaveAbsoluteIndex<Items.Count) and (iSaveAbsoluteIndex>-1) Then
  begin
    Selected:=Items[iSaveAbsoluteIndex];
    Selected.Expanded:=True;
  end;
End;

procedure TXCustomDBTreeView.RefreshTree;
var
  OldChangedEvent: TTVChangedEvent;
Begin
  IF (FDataLink=Nil) or (FDataLink.DataSet=Nil) Then
    Exit;
  Items.BeginUpdate;
  SaveState;
  OldChangedEvent:=OnChange;
  OnChange:=Nil;
  FDataLink.DataSet.Close;
  FDataLink.DataSet.Open;
  OnChange:=OldChangedEvent;
  LoadState;
  Items.EndUpdate;
End;

procedure TXCustomDBTreeView.CreateWnd;
Begin
  inherited;
  IF assigned(Parent) Then
  begin
    Items.BeginUpdate;
    DoBuildTree;
    LoadState;
    Items.EndUpdate;
  end;
End;

procedure TXCustomDBTreeView.DestroyWnd;
var
  OldChangedEvent: TTVChangedEvent;
Begin
  IF assigned(Parent) and (Items.Count>0) Then
    SaveState;
  OldChangedEvent:=OnChange;
  OnChange:=Nil;
  Items.Clear;
  OnChange:=OldChangedEvent;
  inherited;
End;

procedure TXCustomDBTreeView.SaveTree;
var
  i: Integer;
Begin
  For i:=0 To Items.Count-1 Do
  begin
    FNodes.Add(TXTreeNode.Create(Items));
    With Nodes[FNodes.Count-1] Do
    begin
      Text:=Items[i].Text;
      ID:=(Items[i] as TxTreeNode).ID;
      OwnerID:=(Items[i] as TxTreeNode).OwnerID;
      ImageIndex:=Items[i].ImageIndex;
      Data:=Items[i].Data;
    end;
  end;
End;

procedure TXCustomDBTreeView.DoBuildTree;
Begin
  if FUpdateLock or (csLoading in ComponentState) or
    (FDataLink=Nil) or (FDataLink.DataSet=Nil) or
    (FDataFields[0] = '') or (FDataFields[1] = '') or (FDataFields[2] = '') then Exit;
  IF not FDataLink.Active Then
  begin
    Items.Clear;
    Exit;
  end;
  Items.BeginUpdate;
  FUpdateLock:=True;
  Items.Clear;
  With FDataLink.DataSet Do
  begin
    DisableControls;
    First;
    While not EOF Do
    begin
      FNodes.Add(TXTreeNode.Create(Items));
      DoFillNode(Nodes[FNodes.Count-1]);
      With Nodes[FNodes.Count-1] Do
      begin
        Text:=FieldText.AsString;
        ID:=FieldID.AsInteger;
        OwnerID:=FieldOwnerID.AsInteger;
        IF DataFieldImageIndex<>'' Then
          ImageIndex:=FieldImageIndex.AsInteger;
      end;
      Next;
    end;
    EnableControls;
  end;//With FDataLink.DataSet Do
  BuildNode(Nil);
  ClearNodes;
  IF Items.Count>0 Then
  begin
    Selected:=Items[0];
    Selected.Expanded:=True;
  end;
  FUpdateLock:=False;
  if Assigned(FOnBuildTree) then FOnBuildTree(Self);
  Items.EndUpdate;
End;

procedure TXCustomDBTreeView.BuildNode(Node: TXTreeNode);
var
  i: Integer;
  OwnerID: Extended;
  InsNode: TXTreeNode;
begin
  i:=0;
  IF Assigned(Node) Then
    OwnerID:=Node.ID
  Else
    OwnerID:=0;
  While i<FNodes.Count Do
  begin
    IF Nodes[i].OwnerID=OwnerID Then
    begin
      IF OwnerID=0 Then
        InsNode:=Items.AddChild(Nil, Nodes[i].Text) as TXTreeNode
      Else
        InsNode:=Items.AddChild(Node, Nodes[i].Text) as TXTreeNode;
      With InsNode Do
      begin
        ID:=Nodes[i].ID;
        OwnerID:=Nodes[i].OwnerID;
        ImageIndex:=Nodes[i].ImageIndex;
        SelectedIndex:=Nodes[i].ImageIndex;
        Data:=Nodes[i].Data;
      end;
      DoCreateNode(InsNode);
      IF (InsNode.ID>=0) and (InsNode.OwnerID>=0) Then
        BuildNode(InsNode);

      IF FNodes.Count>i Then
      begin
        Nodes[i].Free;
        FNodes.delete(i);
      end;
    end
    Else
      Inc(i);
  end;
end;


END.
