unit XrButton;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Buttons;

type
  TCircleType = (ctLowered,ctRaised);

  TXRSpeedButton = class(TSpeedButton)
  private
    FRadius: Integer;
    FLeftBorder: TCircleType;
    FRightBorder: TCircleType;
    procedure SetRadius(Value: Integer);
  protected
    procedure Paint; override;
  public
  published
    property Radius: Integer read FRadius write SetRadius;
    property LeftBorder: TCircleType read FLeftBorder write FLeftBorder;
    property RightBorder: TCircleType read FRightBorder write FRightBorder;
  end;

function MouseInControl(Control: TControl): Boolean;
procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('X', [TXRSpeedButton]);
end;

function MouseInControl(Control: TControl): Boolean;
var
  P: TPoint;
Begin
  GetCursorPos(P);
  P := Control.ScreenToClient(P);
  With Control.ClientRect do
    Result := (P.X>=Left) and (P.X<=Right) and (P.Y>=Top) and (P.Y<=Bottom);
End;

procedure DrawButton(ARect: TRect;Radius: Integer;Canvas: TCanvas;
  Enabled,Flat,Down,MouseIn: Boolean;LBorder,RBorder: TCircleType);
  procedure DrawUp(Color: TColor);
  begin
    With Canvas,ARect do
    begin
      Pen.Color := Color;
      IF Radius<>0 Then
        IF LBorder=ctRaised Then
          Arc(Left,Top,Left+2*Radius,Bottom,
            Left+Radius,Top,Left+Radius,Bottom)
        Else
          Arc(Left,Top,Left+2*Radius,Bottom,
            Left+Radius,Bottom,Left+Radius,Top)
      Else
      begin
        MoveTo(Left,Top);
        LineTo(Left,Bottom);
      end;
      MoveTo(Left+Radius,Top);
      LineTo(Right-Radius,Top);
    end;
  end; // Up
  procedure DrawDown(Color: TColor);
  begin
    With Canvas,ARect do
    begin
      Pen.Color := Color;
      IF Radius<>0 Then
        IF RBorder=ctRaised Then
          Arc(Right-2*Radius,Top,Right,Bottom,
            Right-Radius,Bottom,Right-Radius,Top)
        Else
          Arc(Right-2*Radius,Top,Right,Bottom,
            Right-Radius,Top,Right-Radius,Bottom)
      Else
      begin
        MoveTo(Right,Top);
        LineTo(Right,Bottom);
      end;
      MoveTo(Right-Radius,Bottom);
      LineTo(Left+Radius,Bottom);
    end;
  end; // Up
Begin // DrawButton
  IF Down Then
  begin
    IF Flat Then DrawUp(clGray) Else DrawUp(clBlack);
    DrawDown(clWhite);
    IF not Flat Then
    begin
      InflateRect(ARect,-1,-1);
      DrawUp(clGray);
      DrawDown(clSilver);
    end;
  end
  Else
  begin
    IF (not Flat) or (MouseIn) Then
      DrawUp(clWhite);
    IF Flat and MouseIn Then
      DrawDown(clGray)
    Else IF not Flat Then
      DrawDown(clBlack);
    IF not Flat Then
    begin
      InflateRect(ARect,-1,-1);
      DrawDown(clGray);
    end;
  end;


End;

procedure TXRSpeedButton.SetRadius(Value: Integer);
Begin
  IF Value=FRadius Then Exit;
  IF Value>Width div 2 Then
    Width := Value*2;
  IF Value>Height div 2 Then
    Height := Value*2;
  FRadius := Value;
  Refresh;
End;

procedure TXRSpeedButton.Paint;
Begin
  DrawButton(ClientRect,FRadius,Canvas,Enabled,Flat, FState=bsDown, MouseInControl(Self),
    FLeftBorder,FRightBorder);
End;

end.
